@@ -2,7 +2,7 @@
  *	HT Editor
  *	analy.cc
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -479,7 +479,7 @@ void CommentList::appendPostComment(int special)
 
 const char *CommentList::getName(UINT i)
 {
-	Object *d = get(i);
+	ht_data *d = get(i);
 	return d ? ((d->object_id()==ATOM_HT_DATA_UINT) ? comment_lookup(((ht_data_uint*)d)->value): ((ht_data_string*)d)->value) : NULL;
 }
 
@@ -657,7 +657,6 @@ int Analyser::load(ht_object_stream *st)
 
 	if (st->get_error()) return st->get_error();
 
-//	st->getObject((Object*)analy_disasm, "analy_disasm");
 	GET_OBJECT(st, analy_disasm);
 	GET_OBJECT(st, disasm);     
 	if (analy_disasm) {
@@ -2241,14 +2240,6 @@ void AnalyDisassembler::init(Analyser *A)
 	initDisasm();
 }
 
-/*
- *
- */
-int  AnalyDisassembler::load(ht_object_stream *st)
-{
-	return st->get_error();
-}
-
 /*
  *
  */
@@ -2266,10 +2257,3 @@ void AnalyDisassembler::initDisasm()
 	}
 }
 
-/*
- *
- */
-void AnalyDisassembler::store(ht_object_stream *f)
-{
-}
-
