@@ -2,7 +2,7 @@
  *	HT Editor
  *	class_analy.cc
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -105,7 +105,7 @@ void ClassAnalyser::beginAnalysis()
 	delete a;
 	if (class_shared->methods) {
 		ClassMethod *cm = NULL;
-		Object *value;
+		ht_data *value;
 		while ((cm = (ClassMethod*)class_shared->methods->enum_next(&value, cm))) {
 			Address *a = createAddress32(cm->start);
 			char buffer2[1024];
@@ -184,7 +184,7 @@ FILEOFS ClassAnalyser::addressToFileofs(Address *Addr)
 /*
  *
  */
-char *ClassAnalyser::getSegmentNameByAddress(Address *Addr)
+const char *ClassAnalyser::getSegmentNameByAddress(Address *Addr)
 {
 	static char sectionname[9];
 	strcpy(sectionname, "test");
