@@ -2,7 +2,7 @@
  *	HT Editor
  *	elf_analy.cc
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -140,14 +140,14 @@ void ElfAnalyser::beginAnalysis()
 
 	/* symbols */
 	if (c32) {
-		for (uint i=1; i<elf_shared->sheaders.count; i++) {
+		for (uint i=1; i < elf_shared->sheaders.count; i++) {
 			if ((elf_shared->sheaders.sheaders32[i].sh_type==ELF_SHT_SYMTAB) || (elf_shared->sheaders.sheaders32[i].sh_type==ELF_SHT_DYNSYM)) {
 				initInsertSymbols(i);
 			}
 		}
 		initInsertFakeSymbols();
 	} else {
-		for (uint i=1; i<elf_shared->sheaders.count; i++) {
+		for (uint i=1; i < elf_shared->sheaders.count; i++) {
 			if ((elf_shared->sheaders.sheaders64[i].sh_type==ELF_SHT_SYMTAB) || (elf_shared->sheaders.sheaders64[i].sh_type==ELF_SHT_DYNSYM)) {
 				initInsertSymbols(i);
 			}
@@ -191,7 +191,7 @@ void ElfAnalyser::initInsertFakeSymbols()
 	sectionAndIdx *key = NULL;
 	ht_data_uint32 *value;
 	while ((key = (sectionAndIdx*)elf_shared->undefined2fakeaddr->enum_next(
-	(Object**)&value, key))) {
+	(ht_data**)&value, key))) {
 		Address *address = createAddress32(value->value);
 		FILEOFS h = elf_shared->sheaders.sheaders32[key->secidx].sh_offset;
 		ELF_SYMBOL32 sym;
@@ -266,14 +266,14 @@ void ElfAnalyser::initInsertSymbols(int shidx)
 					char *label = name;
 					if (!getSymbolByName(label)) {
 						elf32_addr sym_addr = sym.st_value;
-						if (elf_shared->shrelocs && (sym.st_shndx>0) && (sym.st_shndx<elf_shared->sheaders.count)
+						if (elf_shared->shrelocs && sym.st_shndx > 0 && sym.st_shndx < elf_shared->sheaders.count
 						&& elf_shared->shrelocs[sym.st_shndx].relocAddr) {
 							sym_addr += elf_shared->shrelocs[sym.st_shndx].relocAddr;
 						}
 						Address *address = createAddress32(sym_addr);
 						if (validAddress(address, scvalid)) {
 							char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
-
+							if (!demangled) demangled = cplus_demangle_v3(label, DMGL_PARAMS | DMGL_ANSI | DMGL_TYPES);
 							make_valid_name(label, label);
 
 							ht_snprintf(elf_buffer, sizeof elf_buffer, "; function %s (%s)", (demangled) ? demangled : label, bind);
@@ -324,9 +324,9 @@ void ElfAnalyser::initInsertSymbols(int shidx)
 		if (entropy) free(entropy);
 	} else {
 		// FIXME: 64 bit
-		FILEOFS h=elf_shared->sheaders.sheaders64[shidx].sh_offset.lo;
-		FILEOFS sto=elf_shared->sheaders.sheaders64[elf_shared->sheaders.sheaders64[shidx].sh_link].sh_offset.lo;
-		uint symnum=elf_shared->sheaders.sheaders64[shidx].sh_size.lo / sizeof (ELF_SYMBOL64);
+		FILEOFS h = elf_shared->sheaders.sheaders64[shidx].sh_offset.lo;
+		FILEOFS sto = elf_shared->sheaders.sheaders64[elf_shared->sheaders.sheaders64[shidx].sh_link].sh_offset.lo;
+		uint symnum = elf_shared->sheaders.sheaders64[shidx].sh_size.lo / sizeof (ELF_SYMBOL64);
 
 		int *entropy = random_permutation(symnum);
 		for (uint i=0; i<symnum; i++) {
@@ -433,7 +433,7 @@ void ElfAnalyser::initInsertSymbols(int shidx)
  */
 int ElfAnalyser::load(ht_object_stream *f)
 {
-	GET_OBJECT(f, validarea);
+    	GET_OBJECT(f, validarea);
 	return Analyser::load(f);
 }
 
@@ -552,7 +552,7 @@ FILEOFS ElfAnalyser::addressToFileofs(Address *Addr)
  *
  */
 
-char *ElfAnalyser::getSegmentNameByAddress(Address *Addr)
+const char *ElfAnalyser::getSegmentNameByAddress(Address *Addr)
 {
 	static char elf_sectionname[33];
 	elf_section_headers *sections=&elf_shared->sheaders;
