@@ -2,7 +2,7 @@
  *	HT Editor
  *	elf_analy.h
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -31,7 +31,7 @@ class ElfAnalyser: public Analyser {
 public:
 	ht_elf_shared_data 		*elf_shared;
 	ht_streamfile			*file;
-	Area					*validarea;
+	Area				*validarea;
 
 		void		init(ht_elf_shared_data *elf_shared, ht_streamfile *File);
 		int 		load(ht_object_stream *f);
@@ -58,7 +58,7 @@ public:
 	virtual	bool 		validAddress(Address *Addr, tsectype action);
 	virtual	Address		*fileofsToAddress(FILEOFS fileofs);
 	virtual	FILEOFS		addressToFileofs(Address *Addr);
-	virtual	char		*getSegmentNameByAddress(Address *Addr);
+	virtual	const char	*getSegmentNameByAddress(Address *Addr);
 };
 
 #endif
