@@ -2,7 +2,7 @@
  *	HT Editor
  *	le_analy.cc
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -222,7 +222,7 @@ void LEAnalyser::beginAnalysis()
 /*
 	if (le_shared->imports) {
 		ht_tree *t = le_shared->imports;
-		Object *v;
+		ht_data *v;
 		ne_import_rec *imp = NULL;
 		FILEOFS h = le_shared->hdr_ofs + le_shared->hdr.imptab;
 		while ((imp = (ne_import_rec*)t->enum_next(&v, imp))) {
@@ -244,7 +244,7 @@ void LEAnalyser::beginAnalysis()
 	}
 */
 
-/*	virtual Object *enum_next(Object **value, Object *prevkey);
+/*	virtual ht_data *enum_next(ht_data **value, ht_data *prevkey);
 	int import_count = le_shared->imports.funcs->count();
 	for (int i=0; i<import_count; i++) {
 		ht_pe_import_function *f=(ht_pe_import_function *)pe_shared->imports.funcs->get(*(entropy+i));
@@ -409,7 +409,7 @@ Assembler *LEAnalyser::createAssembler()
 /*
  *
  */
-char *LEAnalyser::getSegmentNameByAddress(Address *Addr)
+const char *LEAnalyser::getSegmentNameByAddress(Address *Addr)
 {
 	static char segmentname[16];
 	int i;
