@@ -304,7 +304,7 @@ static int blockop_func_eval(eval_scalar *result, char *name, eval_scalarlist *p
  *	BLOCKOP STRING
  */
 
-class ht_blockop_str_context: public Object {
+class ht_blockop_str_context: public ht_data {
 public:
 	ht_streamfile *file;
 
@@ -329,7 +329,7 @@ public:
 	}
 };
 
-Object *create_blockop_str_context(ht_streamfile *file, FILEOFS ofs, UINT len, UINT size, bool netendian, char *action)
+ht_data *create_blockop_str_context(ht_streamfile *file, FILEOFS ofs, UINT len, UINT size, bool netendian, char *action)
 {
 	ht_blockop_str_context *ctx = new ht_blockop_str_context();
 	ctx->file = file;
@@ -352,7 +352,7 @@ Object *create_blockop_str_context(ht_streamfile *file, FILEOFS ofs, UINT len, U
 		char *s;
 		int p;
 		get_eval_error(&s, &p);
-		throw new ht_io_exception("error evaluating '%s': %s at %d", action, s, p);
+		throw ht_io_exception("error evaluating '%s': %s at %d", action, s, p);
 	}
 
 	ctx->expr_const = blockop_expr_is_const;
@@ -365,7 +365,7 @@ Object *create_blockop_str_context(ht_streamfile *file, FILEOFS ofs, UINT len, U
 }
 
 #define BLOCKOP_STR_MAX_ITERATIONS 1024
-bool blockop_str_process(Object *context, ht_text *progress_indicator)
+bool blockop_str_process(ht_data *context, ht_text *progress_indicator)
 {
 	char status[64];
 	ht_blockop_str_context *ctx = (ht_blockop_str_context*)context;
@@ -379,7 +379,7 @@ bool blockop_str_process(Object *context, ht_text *progress_indicator)
 			
 			ctx->file->seek(ctx->o);
 			if (ctx->file->write(ctx->v.value, s)!=s) {
-				throw new ht_io_exception("blockop_str(): write error at pos %08x, size %08x", ctx->o, s);
+				throw ht_io_exception("blockop_str(): write error at pos %08x, size %08x", ctx->o, s);
 			}
 			ctx->o += s;
 		} else {
@@ -398,7 +398,7 @@ bool blockop_str_process(Object *context, ht_text *progress_indicator)
 				char *s;
 				int p;
 				get_eval_error(&s, &p);
-				throw new ht_io_exception("error evaluating '%s': %s at %d", ctx->action, s, p);
+				throw ht_io_exception("error evaluating '%s': %s at %d", ctx->action, s, p);
 			}
 			scalar_context_str(&r, &sr);
 			scalar_destroy(&r);
@@ -408,7 +408,7 @@ bool blockop_str_process(Object *context, ht_text *progress_indicator)
 
 			ctx->file->seek(ctx->o);
 			if (ctx->file->write(sr.value, s)!=s) {
-				throw new ht_io_exception("blockop_str(): write error at pos %08x, size %08x", ctx->o, s);
+				throw ht_io_exception("blockop_str(): write error at pos %08x, size %08x", ctx->o, s);
 			}
 			string_destroy(&sr);
 			ctx->o += s;
@@ -424,7 +424,7 @@ bool blockop_str_process(Object *context, ht_text *progress_indicator)
  *	BLOCKOP INTEGER
  */
 
-class ht_blockop_int_context: public Object {
+class ht_blockop_int_context: public ht_data {
 public:
 	ht_streamfile *file;
 
@@ -448,7 +448,7 @@ public:
 	}
 };
 
-Object *create_blockop_int_context(ht_streamfile *file, FILEOFS ofs, UINT len, UINT size, endianess endian, char *action)
+ht_data *create_blockop_int_context(ht_streamfile *file, FILEOFS ofs, UINT len, UINT size, endianess endian, char *action)
 {
 	ht_blockop_int_context *ctx = new ht_blockop_int_context();
 	ctx->file = file;
@@ -472,7 +472,7 @@ Object *create_blockop_int_context(ht_streamfile *file, FILEOFS ofs, UINT len, U
 		char *s;
 		int p;
 		get_eval_error(&s, &p);
-		throw new ht_io_exception("error evaluating '%s': %s at %d", action, s, p);
+		throw ht_io_exception("error evaluating '%s': %s at %d", action, s, p);
 	}
 
 	ctx->expr_const = blockop_expr_is_const;
@@ -486,7 +486,7 @@ Object *create_blockop_int_context(ht_streamfile *file, FILEOFS ofs, UINT len, U
 }
 
 #define BLOCKOP_INT_MAX_ITERATIONS	1024
-bool blockop_int_process(Object *context, ht_text *progress_indicator)
+bool blockop_int_process(ht_data *context, ht_text *progress_indicator)
 {
 	ht_blockop_int_context *ctx = (ht_blockop_int_context*)context;
 	char status[64];
@@ -501,7 +501,7 @@ bool blockop_int_process(Object *context, ht_text *progress_indicator)
 			UINT s = ctx->size;
 			if (ctx->o + s > ctx->ofs + ctx->len) s = ctx->ofs + ctx->len - ctx->o;
 			if (ctx->file->write(ibuf, s)!=s) {
-				throw new ht_io_exception("blockop_int(): write error at pos %08x, size %08x", ctx->o, s);
+				throw ht_io_exception("blockop_int(): write error at pos %08x, size %08x", ctx->o, s);
 			}
 			ctx->o += s;
 		} else {
@@ -520,7 +520,7 @@ bool blockop_int_process(Object *context, ht_text *progress_indicator)
 				char *s;
 				int p;
 				get_eval_error(&s, &p);
-				throw new ht_io_exception("error evaluating '%s': %s at %d", ctx->action, s, p);
+				throw ht_io_exception("error evaluating '%s': %s at %d", ctx->action, s, p);
 			}
 			scalar_context_int(&r, &ir);
 			scalar_destroy(&r);
@@ -532,7 +532,7 @@ bool blockop_int_process(Object *context, ht_text *progress_indicator)
 			byte ibuf[4];
 			create_foreign_int(ibuf, ctx->v, ctx->size, ctx->endian);
 			if ((ctx->file->seek(ctx->o) != 0) || (ctx->file->write(ibuf, s)!=s)) {
-				throw new ht_io_exception("blockop_int(): write error at pos %08x, size %08x", ctx->o, s);
+				throw ht_io_exception("blockop_int(): write error at pos %08x, size %08x", ctx->o, s);
 			}
 			ctx->o += s;
 			ctx->i++;
@@ -615,14 +615,14 @@ void blockop_dialog(ht_format_viewer *format, FILEOFS pstart, FILEOFS pend)
 							insert_history_entry((ht_list*)find_atom(HISTATOM_GOTO), addr, NULL);
 						
 							esize = esizes[esize];
-							Object *ctx = NULL;
+							ht_data *ctx = NULL;
 							try {
 								ctx = create_blockop_int_context(file, start, end-start, esize, little_endian, a);
 								if (ctx) {
 									/*bool b = */execute_process(blockop_int_process, ctx);
 								}
-							} catch (ht_exception *e) {
-								errorbox("error: %s", e->what());
+							} catch (const ht_exception &e) {
+								errorbox("error: %s", e.what());
 							}
 							if (ctx) delete ctx;
 							break;
@@ -639,14 +639,14 @@ void blockop_dialog(ht_format_viewer *format, FILEOFS pstart, FILEOFS pend)
 							bin2str(addr, t.end.text, MIN(sizeof addr, t.end.textlen));
 							insert_history_entry((ht_list*)find_atom(HISTATOM_GOTO), addr, NULL);
 
-							Object *ctx = NULL;
+							ht_data *ctx = NULL;
 							try {
 								ctx = create_blockop_str_context(file, start, end-start, esize, little_endian, a);
 								if (ctx) {
 									/*bool b = */execute_process(blockop_str_process, ctx);
 								}
-							} catch (ht_exception *e) {
-								errorbox("error: %s", e->what());
+							} catch (const ht_exception &e) {
+								errorbox("error: %s", e.what());
 							}
 							if (ctx) delete ctx;
 							break;
