@@ -89,7 +89,7 @@ typedef union YYSTYPE {
 	eval_scalarlist scalars;
 } YYSTYPE;
 /* Line 1252 of yacc.c.  */
-#line 93 "y.tab.h"
+#line 93 "evalparse.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
