@@ -29,12 +29,12 @@
 
 #define NEW_TREE_ENUM
 
-int (*qsort_compare_compare_keys)(Object *key_a, Object *key_b);
+int (*qsort_compare_compare_keys)(ht_data *key_a, ht_data *key_b);
 
 int qsort_compare(const void *e1, const void *e2)
 {
-	Object *d1=*(Object **)e1;
-	Object *d2=*(Object **)e2;
+	ht_data *d1=*(ht_data **)e1;
+	ht_data *d2=*(ht_data **)e2;
 	return qsort_compare_compare_keys(d1, d2);
 }
 
@@ -146,14 +146,14 @@ OBJECT_ID ht_data_mem::object_id() const
 
 void ht_tree::init(compare_keys_func_ptr _compare_keys)
 {
-	Object::init();
+	ht_data::init();
 	assert(_compare_keys);
 	compare_keys=_compare_keys;
 }
 
 void ht_tree::done()
 {
-	Object::done();
+	ht_data::done();
 }
 
 void ht_tree::destroy()
@@ -170,32 +170,32 @@ uint ht_tree::count()
 	return 0;
 }
 
-bool ht_tree::del(Object *key)
+bool ht_tree::del(ht_data *key)
 {
 	return false;
 }
 
-Object *ht_tree::enum_next(Object **value, Object *prevkey)
+ht_data *ht_tree::enum_next(ht_data **value, ht_data *prevkey)
 {
 	return NULL;
 }
 
-Object *ht_tree::enum_prev(Object **value, Object *nextkey)
+ht_data *ht_tree::enum_prev(ht_data **value, ht_data *nextkey)
 {
 	return NULL;
 }
 
-Object *ht_tree::get(Object *key)
+ht_data *ht_tree::get(ht_data *key)
 {
 	return NULL;
 }
 
-Object *ht_tree::get_insert(Object *key)
+ht_data *ht_tree::get_insert(ht_data *key)
 {
 	return NULL;
 }
 
-bool ht_tree::insert(Object *key, Object *data)
+bool ht_tree::insert(ht_data *key, ht_data *data)
 {
 	return false;
 }
@@ -257,7 +257,7 @@ uint ht_stree::count()
 	return node_count;
 }
 
-bool ht_stree::del(Object *key)
+bool ht_stree::del(ht_data *key)
 {
 	ht_tree_node *node, *parent;
 	int direction;
@@ -301,7 +301,7 @@ void ht_stree::empty()
 	}
 }
 
-void ht_stree::enum_next_i(ht_tree_node *node, Object *prevkey, ht_tree_node **retv)
+void ht_stree::enum_next_i(ht_tree_node *node, ht_data *prevkey, ht_tree_node **retv)
 {
 	if ((prevkey == NULL) || (compare_keys(prevkey, node->key) < 0)) {
 		*retv = node;
@@ -311,7 +311,7 @@ void ht_stree::enum_next_i(ht_tree_node *node, Object *prevkey, ht_tree_node **r
 	}
 }
 
-void ht_stree::enum_prev_i(ht_tree_node *node, Object *nextkey, ht_tree_node **retv)
+void ht_stree::enum_prev_i(ht_tree_node *node, ht_data *nextkey, ht_tree_node **retv)
 {
 	if ((nextkey == NULL) || (compare_keys(node->key, nextkey) < 0)) {
 		*retv = node;
@@ -321,7 +321,7 @@ void ht_stree::enum_prev_i(ht_tree_node *node, Object *nextkey, ht_tree_node **r
 	}
 }
 
-Object *ht_stree::enum_next(Object **value, Object *prevkey)
+ht_data *ht_stree::enum_next(ht_data **value, ht_data *prevkey)
 {
 #ifdef NEW_TREE_ENUM
 	ht_tree_node *n = NULL, *next = NULL;
@@ -358,7 +358,7 @@ Object *ht_stree::enum_next(Object **value, Object *prevkey)
 #endif
 }
 
-Object *ht_stree::enum_prev(Object **value, Object *nextkey)
+ht_data *ht_stree::enum_prev(ht_data **value, ht_data *nextkey)
 {
 #ifdef NEW_TREE_ENUM
 	ht_tree_node *n = NULL, *prev = NULL;
@@ -419,14 +419,14 @@ void ht_stree::free_skeleton(ht_tree_node *node)
 	node_count--;
 }
 
-Object *ht_stree::get(Object *key)
+ht_data *ht_stree::get(ht_data *key)
 {
 	ht_tree_node *n=get_node_i(key);
 	if (n) return n->value;
 	return NULL;
 }
 
-ht_tree_node *ht_stree::get_node_i(Object *key)
+ht_tree_node *ht_stree::get_node_i(ht_data *key)
 {
 	ht_tree_node *n=root;
 	while (n) {
@@ -438,7 +438,7 @@ ht_tree_node *ht_stree::get_node_i(Object *key)
 	return NULL;
 }
 
-bool ht_stree::get_node_and_parent(Object *key, ht_tree_node **node, ht_tree_node **parent_node, int *direction)
+bool ht_stree::get_node_and_parent(ht_data *key, ht_tree_node **node, ht_tree_node **parent_node, int *direction)
 {
 	ht_tree_node *p=NULL, *n=root;
 	int pc=0;
@@ -474,7 +474,7 @@ ht_tree_node *ht_stree::get_rightmost_node(ht_tree_node *node)
 	return node;
 }
 
-bool ht_stree::insert(Object *key, Object *value)
+bool ht_stree::insert(ht_data *key, ht_data *value)
 {
 //	if ((!key) || (!value)) return false;
 	if (!key) return false;
@@ -535,8 +535,8 @@ int ht_stree::load(ht_object_stream *s)
 	node_count=0;
 	
 	for (uint i=0; i<c; i++) {
-		Object *key=s->get_object(NULL);
-		Object *value=s->get_object(NULL);
+		ht_data *key=s->get_object(NULL);
+		ht_data *value=s->get_object(NULL);
 		if (!key) return 1;
 		if (!value) return 1;
 		insert(key, value);
@@ -559,8 +559,8 @@ void ht_stree::store(ht_object_stream *s)
 	
 	s->putIntDec(count(), 4, NULL);
 	
-	Object *key=NULL;
-	Object *value;
+	ht_data *key=NULL;
+	ht_data *value;
 	while ((key=enum_next(&value, key))) {
 		if (value) {
 			s->putObject(key, NULL);
@@ -663,7 +663,7 @@ uint ht_dtree::count()
 	return node_count-dead_node_count;
 }
 
-bool ht_dtree::del(Object *key)
+bool ht_dtree::del(ht_data *key)
 {
 	ht_tree_node *n=get_node_i(key);
 	if ((n) && (n->value)) {
@@ -679,7 +679,7 @@ bool ht_dtree::del(Object *key)
 	return false;
 }
 
-Object *ht_dtree::enum_next(Object **value, Object *prevkey)
+ht_data *ht_dtree::enum_next(ht_data **value, ht_data *prevkey)
 {
 #ifdef NEW_TREE_ENUM
 	ht_tree_node *n, *next;
@@ -712,7 +712,7 @@ Object *ht_dtree::enum_next(Object **value, Object *prevkey)
 	return NULL;
 #else
 	ht_tree_node *n;
-	Object *k = prevkey;
+	ht_data *k = prevkey;
 	while (1) {
 		n = NULL;
 		if (root) enum_next_i(root, k, &n);
@@ -727,7 +727,7 @@ Object *ht_dtree::enum_next(Object **value, Object *prevkey)
 #endif
 }
 
-Object *ht_dtree::enum_prev(Object **value, Object *nextkey)
+ht_data *ht_dtree::enum_prev(ht_data **value, ht_data *nextkey)
 {
 #ifdef NEW_TREE_ENUM
 	ht_tree_node *n, *prev;
@@ -760,7 +760,7 @@ Object *ht_dtree::enum_prev(Object **value, Object *nextkey)
 	return NULL;
 #else
 	ht_tree_node *n;
-	Object *k = nextkey;
+	ht_data *k = nextkey;
 	while (1) {
 		n = NULL;
 		if (root) enum_prev_i(root, k, &n);
@@ -775,7 +775,7 @@ Object *ht_dtree::enum_prev(Object **value, Object *nextkey)
 #endif
 }
 
-bool ht_dtree::insert(Object *key, Object *value)
+bool ht_dtree::insert(ht_data *key, ht_data *value)
 {
 	if ((!key) || (!value)) return false;
 	ht_tree_node **n=&root;
@@ -812,13 +812,13 @@ bool ht_dtree::insert(Object *key, Object *value)
 
 void ht_list::init(compare_keys_func_ptr _compare_keys)
 {
-	Object::init();
+	ht_data::init();
 	compare_keys=_compare_keys;
 }
 
 void ht_list::done()
 {
-	Object::done();
+	ht_data::done();
 }
 
 void ht_list::destroy()
@@ -826,7 +826,7 @@ void ht_list::destroy()
 	done();
 }
 
-void ht_list::append(Object *data)
+void ht_list::append(ht_data *data)
 {
 }
 
@@ -839,7 +839,7 @@ void ht_list::copy_to(uint i, uint count, ht_list *destlist)
 {
 	uint j=0;
 	while (count--) {
-		Object *w=get(i+j);
+		ht_data *w=get(i+j);
 		if (j) {
 			destlist->insert_after(w, j-1);
 		} else {
@@ -872,25 +872,25 @@ void ht_list::empty()
 {
 }
 
-uint ht_list::find(Object *data)
+uint ht_list::find(ht_data *data)
 {
 	return LIST_UNDEFINED;
 }
 
-Object *ht_list::get(uint i)
+ht_data *ht_list::get(uint i)
 {
 	return NULL;
 }
 
-void ht_list::insert(Object *data)
+void ht_list::insert(ht_data *data)
 {
 }
 
-void ht_list::insert_after(Object *data, uint i)
+void ht_list::insert_after(ht_data *data, uint i)
 {
 }
 
-void ht_list::insert_before(Object *data, uint i)
+void ht_list::insert_before(ht_data *data, uint i)
 {
 }
 
@@ -902,11 +902,11 @@ void ht_list::move_multiple(uint source, uint dest, uint count)
 {
 }
 
-void ht_list::prepend(Object *data)
+void ht_list::prepend(ht_data *data)
 {
 }
 
-Object *ht_list::remove(uint i)
+ht_data *ht_list::remove(uint i)
 {
 	return NULL;
 }
@@ -920,7 +920,7 @@ bool ht_list::remove_multiple(uint i, uint count)
 	return b;
 }
 
-bool ht_list::set(uint i, Object *data)
+bool ht_list::set(uint i, ht_data *data)
 {
 	return false;
 }
@@ -944,7 +944,7 @@ void ht_clist::init(compare_keys_func_ptr compare_keys)
 {
 	ht_list::init(compare_keys);
 	c_size=HT_CLIST_ENTRY_COUNT_START;
-	items=(Object**)malloc(c_size * sizeof (Object*));
+	items=(ht_data**)malloc(c_size * sizeof (ht_data*));
 	c_entry_count=0;
 }
 
@@ -960,7 +960,7 @@ void ht_clist::destroy()
 	done();
 }
 
-void ht_clist::append(Object *data)
+void ht_clist::append(ht_data *data)
 {
 	insert_before(data, c_entry_count);
 }
@@ -1027,13 +1027,13 @@ void ht_clist::extend_list()
 {
 	c_size *= HT_CLIST_ENTRY_COUNT_EXT_NUM;
 	c_size /= HT_CLIST_ENTRY_COUNT_EXT_DEN;
-	Object **new_items=(Object**)malloc(c_size * sizeof (Object*));
-	memmove(new_items, items, c_entry_count * sizeof (Object*));
+	ht_data **new_items=(ht_data**)malloc(c_size * sizeof (ht_data*));
+	memmove(new_items, items, c_entry_count * sizeof (ht_data*));
 	free(items);
 	items=new_items;
 }
 
-uint ht_clist::find(Object *data)
+uint ht_clist::find(ht_data *data)
 {
 	if (compare_keys) {
 		for (uint i=0; i<c_entry_count; i++) {
@@ -1043,23 +1043,23 @@ uint ht_clist::find(Object *data)
 	return LIST_UNDEFINED;
 }
 
-Object *ht_clist::get(uint i)
+ht_data *ht_clist::get(uint i)
 {
 	if (i<c_entry_count) return items[i];
 	return NULL;
 }
 
-void ht_clist::insert(Object *data)
+void ht_clist::insert(ht_data *data)
 {
 	append(data);
 }
 
-void ht_clist::insert_after(Object *data, uint i)
+void ht_clist::insert_after(ht_data *data, uint i)
 {
 	insert_before(data, i+1);
 }
 
-void ht_clist::insert_before(Object *data, uint i)
+void ht_clist::insert_before(ht_data *data, uint i)
 {
 	uint hi= i>c_entry_count ? i+1 : c_entry_count+1;
 	while (hi>=c_size) extend_list();
@@ -1075,7 +1075,7 @@ void ht_clist::insert_before(Object *data, uint i)
 int  ht_clist::load(ht_object_stream *s)
 {
 	c_size=HT_CLIST_ENTRY_COUNT_START;
-	items=(Object**)malloc(c_size * sizeof (Object*));
+	items=(ht_data**)malloc(c_size * sizeof (ht_data*));
 	c_entry_count=0;
 	
 	void *d=find_atom(s->getIntHex(4, NULL));
@@ -1084,7 +1084,7 @@ int  ht_clist::load(ht_object_stream *s)
 	
 	int c=s->getIntDec(4, "item_count");
 	for (int i=0; i<c; i++) {
-		Object *d=s->getObject("item");
+		ht_data *d=s->getObject("item");
 		if (s->get_error()) break;
 		prepend(d);
 	}
@@ -1094,7 +1094,7 @@ int  ht_clist::load(ht_object_stream *s)
 void ht_clist::move(uint source, uint dest)
 {
 	if (dest<=c_entry_count) {
-		Object *src=get(source);
+		ht_data *src=get(source);
 		memmove(items+source, items+source+1, sizeof items[0] * (c_entry_count-source-1));
 		memmove(items+dest+1, items+dest, sizeof items[0] * (c_entry_count-dest-1));
 		items[dest]=src;
@@ -1115,22 +1115,22 @@ OBJECT_ID ht_clist::object_id() const
 	return ATOM_HT_CLIST;
 }
 
-void ht_clist::prepend(Object *data)
+void ht_clist::prepend(ht_data *data)
 {
 	insert_before(data, 0);
 }
 
-Object *ht_clist::remove(uint i)
+ht_data *ht_clist::remove(uint i)
 {
 	if (i<c_entry_count) {
-		Object *d=items[i];
+		ht_data *d=items[i];
 		do_remove(i);
 		return d;
 	}
 	return NULL;
 }
 
-bool ht_clist::set(uint i, Object *data)
+bool ht_clist::set(uint i, ht_data *data)
 {
 	while (i>=c_entry_count) append(NULL);
 	do_free(i);
@@ -1168,13 +1168,13 @@ bool ht_clist::qsort_i(uint _l, uint _r)
 	int m=(l+r) / 2;
 	int origl=l;
 	int origr=r;
-	Object *c;
+	ht_data *c;
 	c=items[m];
 	do {
 		while ((l<m) && compare_keys(items[l], c) < 0) l++;
 		while ((r>m) && compare_keys(items[r], c) > 0) r--;
 		if (l < r) {
-			Object *t;
+			ht_data *t;
 			t=items[l];
 			items[l]=items[r];
 			items[r]=t;
@@ -1205,12 +1205,12 @@ void ht_sorted_list::done()
 	ht_clist::done();
 }
 
-void ht_sorted_list::append(Object *data)
+void ht_sorted_list::append(ht_data *data)
 {
 	insert(data);
 }
 
-uint ht_sorted_list::find(Object *data)
+uint ht_sorted_list::find(ht_data *data)
 {
 	if (compare_keys) {
 		HT_ERROR("function untested !");
@@ -1226,7 +1226,7 @@ uint ht_sorted_list::find(Object *data)
 	return LIST_UNDEFINED;
 }
 
-void ht_sorted_list::insert(Object *data)
+void ht_sorted_list::insert(ht_data *data)
 {
 	if (c_entry_count) {
 		int l=0, r=c_entry_count-1;
@@ -1243,12 +1243,12 @@ void ht_sorted_list::insert(Object *data)
 	}
 }
 
-void ht_sorted_list::insert_after(Object *data, uint i)
+void ht_sorted_list::insert_after(ht_data *data, uint i)
 {
 	insert(data);
 }
 
-void ht_sorted_list::insert_before(Object *data, uint i)
+void ht_sorted_list::insert_before(ht_data *data, uint i)
 {
 	insert(data);
 }
@@ -1263,12 +1263,12 @@ void ht_sorted_list::move_multiple(uint source, uint dest, uint count)
 	// do nothing
 }
 
-void ht_sorted_list::prepend(Object *data)
+void ht_sorted_list::prepend(ht_data *data)
 {
 	insert(data);
 }
 
-bool ht_sorted_list::set(uint i, Object *data)
+bool ht_sorted_list::set(uint i, ht_data *data)
 {
 	insert(data);
 	return true;
@@ -1278,17 +1278,17 @@ bool ht_sorted_list::set(uint i, Object *data)
  *	CLASS ht_stack
  */
 
-Object *ht_stack::pop()
+ht_data *ht_stack::pop()
 {
 	if (c_entry_count) {
-		Object *d=get(c_entry_count-1);
+		ht_data *d=get(c_entry_count-1);
 		do_remove(c_entry_count-1);
 		return d;
 	}
 	return NULL;
 }
 
-void ht_stack::push(Object *data)
+void ht_stack::push(ht_data *data)
 {
 	append(data);
 }
@@ -1297,27 +1297,27 @@ void ht_stack::push(Object *data)
  *	CLASS ht_queue
  */
 
-void	ht_queue::enqueue(Object *data)
+void	ht_queue::enqueue(ht_data *data)
 {
 	append(data);
 }
 
-Object *ht_queue::dequeue()
+ht_data *ht_queue::dequeue()
 {
 	if (c_entry_count) {
-		Object *d=get(0);
+		ht_data *d=get(0);
 		do_remove(0);
 		return d;
 	}
 	return NULL;
 }
 
-Object *ht_queue::pop()
+ht_data *ht_queue::pop()
 {
 	return dequeue();
 }
 
-void	ht_queue::push(Object *data)
+void	ht_queue::push(ht_data *data)
 {
 	enqueue(data);
 }
@@ -1341,12 +1341,12 @@ char *matchhash(int value, int_hash *hash_table)
  *	compare procedures
  */
 
-int compare_keys_ht_data(Object *key_a, Object *key_b)
+int compare_keys_ht_data(ht_data *key_a, ht_data *key_b)
 {
 	return key_a->compareTo(key_b);
 }
 
-int compare_keys_int(Object *key_a, Object *key_b)
+int compare_keys_int(ht_data *key_a, ht_data *key_b)
 {
 	int a=((ht_data_uint*)key_a)->value;
 	int b=((ht_data_uint*)key_b)->value;
@@ -1354,7 +1354,7 @@ int compare_keys_int(Object *key_a, Object *key_b)
 	return 0;
 }
 
-int compare_keys_uint(Object *key_a, Object *key_b)
+int compare_keys_uint(ht_data *key_a, ht_data *key_b)
 {
 	uint a=((ht_data_uint*)key_a)->value;
 	uint b=((ht_data_uint*)key_b)->value;
