@@ -2,7 +2,7 @@
  *	HT Editor
  *	htelfimg.cc
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -23,13 +23,12 @@
 #include "htpal.h"
 #include "htstring.h"
 #include "formats.h"
-#include "snprintf.h"
 #include "tools.h"
 
 #include "elfstruc.h"
 #include "elf_analy.h"
 
-ht_view *htelfimage_init(bounds *b, ht_streamfile *file, ht_format_group *group)
+static ht_view *htelfimage_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 {
 	ht_elf_shared_data *elf_shared=(ht_elf_shared_data *)group->get_shared_data();
 
@@ -56,7 +55,7 @@ ht_view *htelfimage_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 
 	v->attachInfoline(head);
 
-/* find lowest/highest address */
+	/* find lowest/highest address */
 	Address *low=NULL;
 	Address *high=NULL;
 	switch (elf_shared->ident.e_ident[ELF_EI_CLASS]) {
@@ -132,7 +131,7 @@ ht_view *htelfimage_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 			break;
 		}
 	}
-	
+
 	v->gotoAddress(tmpaddr, NULL);
 	delete tmpaddr;
 
