@@ -2,7 +2,7 @@
  *	HT Editor
  *	htelfshs.cc
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -24,10 +24,9 @@
 #include "htelfshs.h"
 #include "httag.h"
 #include "formats.h"
+#include "snprintf.h"
 
-#include <stdlib.h>
-
-ht_mask_ptable elfsectionheader32[]=
+static ht_mask_ptable elfsectionheader32[]=
 {
 	{"name string index",	STATICTAG_EDIT_DWORD_VE("00000000")},
 	{"type",				STATICTAG_EDIT_DWORD_VE("00000004")" ("STATICTAG_DESC_DWORD_VE("00000004", ATOM_ELF_SH_TYPE_STR)")"},
@@ -42,7 +41,7 @@ ht_mask_ptable elfsectionheader32[]=
 	{0, 0}
 };
 
-ht_mask_ptable elfsectionheader64[]=
+static ht_mask_ptable elfsectionheader64[]=
 {
 	{"name string index",	STATICTAG_EDIT_DWORD_VE("00000000")},
 	{"type",				STATICTAG_EDIT_DWORD_VE("00000004")" ("STATICTAG_DESC_DWORD_VE("00000004", ATOM_ELF_SH_TYPE_STR)")"},
@@ -57,7 +56,7 @@ ht_mask_ptable elfsectionheader64[]=
 	{0, 0}
 };
 
-int_hash elf_sh_type[] =
+static int_hash elf_sh_type[] =
 {
 	{ELF_SHT_NULL, 		"null"},
 	{ELF_SHT_PROGBITS,	"progbits"},
@@ -77,7 +76,7 @@ int_hash elf_sh_type[] =
 	{0, 0}
 };
 
-ht_tag_flags_s elf_sh_flags[] =
+static ht_tag_flags_s elf_sh_flags[] =
 {
 	{0,  "[00] writable"},
 	{1,  "[01] alloc"},
@@ -91,7 +90,7 @@ ht_tag_flags_s elf_sh_flags[] =
 	{0, 0}
 };
 
-ht_view *htelfsectionheaders_init(bounds *b, ht_streamfile *file, ht_format_group *group)
+static ht_view *htelfsectionheaders_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 {
 	ht_elf_shared_data *elf_shared=(ht_elf_shared_data *)group->get_shared_data();
 
@@ -110,7 +109,7 @@ ht_view *htelfsectionheaders_init(bounds *b, ht_streamfile *file, ht_format_grou
 		m->init(file, 0);
 
 		char info[128];
-		sprintf(info, "* ELF section headers at offset %08x", h);
+		ht_snprintf(info, sizeof info, "* ELF section headers at offset %08x", h);
 
 		m->add_mask(info);
 
@@ -119,55 +118,57 @@ ht_view *htelfsectionheaders_init(bounds *b, ht_streamfile *file, ht_format_grou
 		elf_shared->shnames = (char**)malloc(elf_shared->sheaders.count * sizeof *elf_shared->shnames);
 		FILEOFS so=elf_shared->sheaders.sheaders32[elf_shared->header32.e_shstrndx].sh_offset;
 		for (UINT i=0; i<elf_shared->sheaders.count; i++) {
-			ht_mask_sub *n=new ht_mask_sub();
-			n->init(file, i);
-
-			file->seek(so+elf_shared->sheaders.sheaders32[i].sh_name);
-			char *s = fgetstrz(file);
+			char *s;
+			if (file->seek(so+elf_shared->sheaders.sheaders32[i].sh_name)
+			|| !((s = fgetstrz(file)))) s = "?";
 			char t[1024];
-			sprintf(t, "section %d: %s", i, s);
+			ht_snprintf(t, sizeof t, "section %d: %s", i, s);
 			elf_shared->shnames[i] = s;
 
+			ht_mask_sub *n = new ht_mask_sub();
+			n->init(file, i);
+
 			n->add_staticmask_ptable(elfsectionheader32, h+i*elf_shared->header32.e_shentsize, elf_bigendian);
 
-			ht_collapsable_sub *cn=new ht_collapsable_sub();
+			ht_collapsable_sub *cn = new ht_collapsable_sub();
 			cn->init(file, n, 1, t, 1);
 
 			v->insertsub(cn);
 		}
 	} else if (elf_shared->ident.e_ident[ELF_EI_CLASS]==ELFCLASS64) {
-		v=new ht_uformat_viewer();
+		v = new ht_uformat_viewer();
 		v->init(b, DESC_ELF_SECTION_HEADERS, VC_EDIT, file, group);
 
 		register_atom(ATOM_ELF_SH_TYPE, elf_sh_type);
 		register_atom(ATOM_ELF_SH_FLAGS, elf_sh_flags);
 
-/* FIXME: 64-bit */
+		/* FIXME: 64-bit */
 		FILEOFS h = elf_shared->header64.e_shoff.lo;
 
 		ht_mask_sub *m=new ht_mask_sub();
 		m->init(file, 0);
 
 		char info[128];
-		sprintf(info, "* ELF section headers at offset %08x", h);
+		ht_snprintf(info, sizeof info, "* ELF section headers at offset %08x", h);
 
 		m->add_mask(info);
 
 		v->insertsub(m);
 
 		elf_shared->shnames=(char**)malloc(elf_shared->sheaders.count * sizeof *elf_shared->shnames);
-/* FIXME: 64-bit */
+		/* FIXME: 64-bit */
 		FILEOFS so=elf_shared->sheaders.sheaders64[elf_shared->header64.e_shstrndx].sh_offset.lo;
 		for (UINT i=0; i<elf_shared->sheaders.count; i++) {
+			char *s;
+			if (file->seek(so+elf_shared->sheaders.sheaders64[i].sh_name)
+			|| !((s=fgetstrz(file)))) s = "?";
+			char t[1024];
+			ht_snprintf(t, sizeof t, "section %d: %s", i, s);
+			elf_shared->shnames[i]=s;
+
 			ht_mask_sub *n=new ht_mask_sub();
 			n->init(file, i);
 
-			file->seek(so+elf_shared->sheaders.sheaders64[i].sh_name);
-			char *s=fgetstrz(file);
-			char t[1024];
-			sprintf(t, "section %d: %s", i, s);
-			elf_shared->shnames[i]=s;
-
 			n->add_staticmask_ptable(elfsectionheader64, h+i*elf_shared->header64.e_shentsize, elf_bigendian);
 
 			ht_collapsable_sub *cn=new ht_collapsable_sub();
