@@ -2,7 +2,7 @@
  *	HT Editor
  *	htleent.cc
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -24,12 +24,13 @@
 #include "htlehead.h"
 #include "httag.h"
 #include "formats.h"
+#include "snprintf.h"
 
 #include "lestruct.h"
 
 /* entry bundle */
 
-ht_tag_flags_s le_entry_bundle_flags[] =
+static ht_tag_flags_s le_entry_bundle_flags[] =
 {
 	{-1, "LE - entrypoint bundle flags"},
 	{0,  "[00] valid"},
@@ -37,7 +38,7 @@ ht_tag_flags_s le_entry_bundle_flags[] =
 	{0, 0}
 };
 
-ht_mask_ptable le_entry_bundle_header[]=
+static ht_mask_ptable le_entry_bundle_header[]=
 {
 	{"number of entries",	STATICTAG_EDIT_BYTE("00000000")},
 	{"bundle flags",		STATICTAG_EDIT_BYTE("00000001")" "STATICTAG_FLAGS("00000001", ATOM_LE_ENTRY_BUNDLE_FLAGS_STR)},
@@ -47,7 +48,7 @@ ht_mask_ptable le_entry_bundle_header[]=
 
 /* entry */
 
-ht_tag_flags_s le_entry_flags[] =
+static ht_tag_flags_s le_entry_flags[] =
 {
 	{-1, "LE - entrypoint flags"},
 	{0,  "[00] exported"},
@@ -55,21 +56,21 @@ ht_tag_flags_s le_entry_flags[] =
 	{0, 0}
 };
 
-ht_mask_ptable le_entry16[]=
+static ht_mask_ptable le_entry16[]=
 {
 	{"flags",		STATICTAG_EDIT_BYTE("00000000")" "STATICTAG_FLAGS("00000000", ATOM_LE_ENTRY_FLAGS_STR)},
 	{"offset",	STATICTAG_EDIT_WORD_LE("00000001")},
 	{0, 0}
 };
 
-ht_mask_ptable le_entry32[]=
+static ht_mask_ptable le_entry32[]=
 {
 	{"flags",		STATICTAG_EDIT_BYTE("00000000")" "STATICTAG_FLAGS("00000000", ATOM_LE_ENTRY_FLAGS_STR)},
 	{"offset",	STATICTAG_EDIT_DWORD_LE("00000001")},
 	{0, 0}
 };
 
-ht_view *htleentrypoints_init(bounds *b, ht_streamfile *file, ht_format_group *group)
+static ht_view *htleentrypoints_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 {
 	ht_le_shared_data *le_shared=(ht_le_shared_data *)group->get_shared_data();
 
@@ -82,7 +83,7 @@ ht_view *htleentrypoints_init(bounds *b, ht_streamfile *file, ht_format_group *g
 	register_atom(ATOM_LE_ENTRY_BUNDLE_FLAGS, le_entry_bundle_flags);
 	char info[128];
 	
-	sprintf(info, "* LE entry header at offset %08x", h+le_shared->hdr.enttab);
+	ht_snprintf(info, sizeof info, "* LE entry header at offset %08x", h+le_shared->hdr.enttab);
 
 /* FIXME: false */
 	bool le_bigendian = false;
@@ -114,7 +115,7 @@ ht_view *htleentrypoints_init(bounds *b, ht_streamfile *file, ht_format_group *g
 				m->add_staticmask_ptable(le_entry16, o, le_bigendian);
 				o+=1+2;
 			}
-			sprintf(t, "--- entry %d (%s) ---", i+1, flags_str);
+			ht_snprintf(t, sizeof t, "--- entry %d (%s) ---", i+1, flags_str);
 
 			ht_collapsable_sub *cs=new ht_collapsable_sub();
 			cs->init(file, m, 1, t, 1);
