@@ -2,7 +2,7 @@
  *	HT Editor
  *	htleobj.cc
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -25,13 +25,14 @@
 #include "httag.h"
 #include "htstring.h"
 #include "formats.h"
+#include "snprintf.h"
 
 #include "lestruct.h"
 
 #include <stdlib.h>
 #include <string.h>
 
-ht_mask_ptable leobj[]=
+static ht_mask_ptable leobj[]=
 {
 	{"virtual size",			STATICTAG_EDIT_DWORD_LE("00000000")},
 	{"relocation base address",	STATICTAG_EDIT_DWORD_LE("00000004")},
@@ -42,7 +43,7 @@ ht_mask_ptable leobj[]=
 	{0, 0}
 };
 
-ht_tag_flags_s le_objflags[] =
+static ht_tag_flags_s le_objflags[] =
 {
 	{0,  "[00] readable"},
 	{1,  "[01] writable"},
@@ -63,7 +64,7 @@ ht_tag_flags_s le_objflags[] =
 	{0, 0}
 };
 
-ht_view *htleobjects_init(bounds *b, ht_streamfile *file, ht_format_group *group)
+static ht_view *htleobjects_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 {
 	ht_le_shared_data *le_shared=(ht_le_shared_data *)group->get_shared_data();
 
@@ -79,15 +80,15 @@ ht_view *htleobjects_init(bounds *b, ht_streamfile *file, ht_format_group *group
 	bool le_bigendian = false;
 
 	char t[64];
-	sprintf(t, "* LE object headers at offset %08x", h+le_shared->hdr.objtab);
+	ht_snprintf(t, sizeof t, "* LE object headers at offset %08x", h+le_shared->hdr.objtab);
 	m->add_mask(t);
 
 	v->insertsub(m);
-	
+
 	for (dword i=0; i<le_shared->hdr.objcnt; i++) {
 		m=new ht_mask_sub();
 		m->init(file, i);
-		
+
 		char n[5];
 		m->add_staticmask_ptable(leobj, h+le_shared->hdr.objtab+i*24, le_bigendian);
 		
@@ -96,8 +97,8 @@ ht_view *htleobjects_init(bounds *b, ht_streamfile *file, ht_format_group *group
 
 		bool use32=le_shared->objmap.header[i].flags & LE_OBJECT_FLAG_USE32;
 
-		sprintf(t, "--- object %d USE%d: %s ---", i+1, use32 ? 32 : 16, (char*)&n);
-		
+		ht_snprintf(t, sizeof t, "--- object %d USE%d: %s ---", i+1, use32 ? 32 : 16, (char*)&n);
+
 		ht_collapsable_sub *cs=new ht_collapsable_sub();
 		cs->init(file, m, 1, t, 1);
 		v->insertsub(cs);
@@ -111,4 +112,3 @@ format_viewer_if htleobjects_if = {
 	htleobjects_init,
 	0
 };
-
