@@ -96,7 +96,7 @@ bool execute_submenu(int x, int y, ht_context_menu *m)
 
 void ht_context_menu::init(char *Name)
 {
-	Object::init();
+	ht_data::init();
 	name = ht_strdup(Name);
 	shortcut = shortcut_str(name);
 	width = 0;
@@ -105,7 +105,7 @@ void ht_context_menu::init(char *Name)
 void ht_context_menu::done()
 {
 	free(name);
-	Object::done();
+	ht_data::done();
 }
 
 int ht_context_menu::count()
