@@ -2,7 +2,7 @@
  *	HT Editor
  *	htneobj.cc
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -26,22 +26,23 @@
 #include "httag.h"
 #include "htstring.h"
 #include "formats.h"
+#include "snprintf.h"
 
 #include "nestruct.h"
 
 #include <stdlib.h>
 #include <string.h>
 
-ht_mask_ptable neobj[]=
+static ht_mask_ptable neobj[]=
 {
 	{"offset",	STATICTAG_EDIT_WORD_LE("00000000")},
-	{"size",		STATICTAG_EDIT_WORD_LE("00000002")},
-	{"flags",		STATICTAG_EDIT_WORD_LE("00000004")" "STATICTAG_FLAGS("00000004", ATOM_NE_SEGFLAGS_STR)},
+	{"size",	STATICTAG_EDIT_WORD_LE("00000002")},
+	{"flags",	STATICTAG_EDIT_WORD_LE("00000004")" "STATICTAG_FLAGS("00000004", ATOM_NE_SEGFLAGS_STR)},
 	{"minalloc",	STATICTAG_EDIT_WORD_LE("00000006")},
 	{0, 0}
 };
 
-ht_tag_flags_s ne_segflags[] =
+static ht_tag_flags_s ne_segflags[] =
 {
 	{0,  "[00] data"},
 	{1,  "[01] reserved"},
@@ -62,7 +63,7 @@ ht_tag_flags_s ne_segflags[] =
 	{0, 0}
 };
 
-ht_view *htnesegments_init(bounds *b, ht_streamfile *file, ht_format_group *group)
+static ht_view *htnesegments_init(bounds *b, ht_streamfile *file, ht_format_group *group)
 {
 	ht_ne_shared_data *ne_shared=(ht_ne_shared_data *)group->get_shared_data();
 
@@ -76,7 +77,7 @@ ht_view *htnesegments_init(bounds *b, ht_streamfile *file, ht_format_group *grou
 
 	char t[64];
 
-	sprintf(t, "* NE segment table at offset %08x", h+ne_shared->hdr.segtab);
+	ht_snprintf(t, sizeof t, "* NE segment table at offset %08x", h+ne_shared->hdr.segtab);
 	m->add_mask(t);
 	m->add_mask("note: 0 means 65536 for segment size and minalloc");
 
@@ -85,7 +86,7 @@ ht_view *htnesegments_init(bounds *b, ht_streamfile *file, ht_format_group *grou
 		NE_SEGMENT s;
 		file->read(&s, sizeof s);
 		create_host_struct(&s, NE_SEGMENT_struct, little_endian);
-		sprintf(t, "--- segment %d (%s) ---", i+1, (s.flags & NE_DATA) ? "data" : "code");
+		ht_snprintf(t, sizeof t, "--- segment %d (%s) ---", i+1, (s.flags & NE_DATA) ? "data" : "code");
 		m->add_mask(t);
 		m->add_staticmask_ptable(neobj, h+ne_shared->hdr.segtab+i*8, false);
 	}
@@ -98,4 +99,3 @@ format_viewer_if htnesegments_if = {
 	htnesegments_init,
 	0
 };
-
