@@ -28,7 +28,7 @@
 
 extern format_viewer_if htpeil_if;
 
-class ht_il_metadata_entry: public Object {
+class ht_il_metadata_entry: public ht_data {
 public:
 	char *name;
 	dword offset;
