@@ -44,7 +44,7 @@ UINT lastreplacemodeid=0;
 
 typedef ht_search_request* (*create_request_func)(search_pos *ret_start, search_pos *ret_end, ht_view *form, ht_format_viewer *format, UINT search_class);
 
-typedef Object* (*create_replace_context_func)(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen);
+typedef ht_data* (*create_replace_context_func)(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen);
 
 struct ht_search_method {
 	char *name;
@@ -71,14 +71,14 @@ bool test_str_to_ofs(FILEOFS *ofs, byte *str, UINT strlen, ht_format_viewer *for
 {
 #define TEST_STR_TO_OFS_MAXSTRLEN       128
 	if (strlen>TEST_STR_TO_OFS_MAXSTRLEN) {
-		throw new ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_OFS_MAXSTRLEN);
+		throw ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_OFS_MAXSTRLEN);
 		return false;
 	}
 	if (strlen>0) {
 		char s[TEST_STR_TO_OFS_MAXSTRLEN+1];
 		bin2str(s, str, strlen);
 		if (!format->string_to_offset(s, ofs)) {
-			throw new ht_io_exception("%s: invalid expression: '%s'", desc, s);
+			throw ht_io_exception("%s: invalid expression: '%s'", desc, s);
 			return false;
 		}
 	}
@@ -89,14 +89,14 @@ bool test_str_to_pos(viewer_pos *pos, byte *str, UINT strlen, ht_format_viewer *
 {
 #define TEST_STR_TO_POS_MAXSTRLEN      128
 	if (strlen>TEST_STR_TO_POS_MAXSTRLEN) {
-		throw new ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_POS_MAXSTRLEN);
+		throw ht_io_exception("%s: expression too long (len %d, max %d)", desc, strlen, TEST_STR_TO_POS_MAXSTRLEN);
 		return false;
 	}
 	if (strlen>0) {
 		char s[TEST_STR_TO_POS_MAXSTRLEN+1];
 		bin2str(s, str, strlen);
 		if (!format->string_to_pos(s, pos)) {
-			throw new ht_io_exception("%s: invalid expression: '%s'", desc, s);
+			throw ht_io_exception("%s: invalid expression: '%s'", desc, s);
 			return false;
 		}
 	}
@@ -123,7 +123,7 @@ ht_search_request* create_request_hexascii(search_pos *start, search_pos *end, h
 	ht_fxbin_search_request *request;
 	
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "hex/ascii");
+		throw ht_io_exception("%s: string is empty", "hex/ascii");
 	}
 	if (test_str_to_ofs(&start->offset, d.start.text, d.start.textlen, format, "start-offset")
 	&& test_str_to_ofs(&end->offset, d.end.text, d.end.textlen, format, "end-offset")) {
@@ -143,7 +143,7 @@ void create_desc_hexascii(char *buf, int buflen, ht_view *f)
 	form->databuf_get(&d, sizeof d);
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "hex/ascii");
+		throw ht_io_exception("%s: string is empty", "hex/ascii");
 	}
 
 	char *b = buf;
@@ -186,7 +186,7 @@ ht_search_request* create_request_evalstr(search_pos *start, search_pos *end, ht
 	ht_fxbin_search_request *request = NULL;
 		
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "eval str");
+		throw ht_io_exception("%s: string is empty", "eval str");
 	} else if (d.str.textlen<=EVALSTR_MAXSTRLEN) {
 		char strbuf[EVALSTR_MAXSTRLEN+1];
 		bin2str(strbuf, d.str.text, d.str.textlen);
@@ -206,10 +206,10 @@ ht_search_request* create_request_evalstr(search_pos *start, search_pos *end, ht
 			char *str;
 			int pos;
 			get_eval_error(&str, &pos);
-			throw new ht_io_exception("eval error at pos %d: %s", pos, str);
+			throw ht_io_exception("eval error at pos %d: %s", pos, str);
 		}
 	} else {
-		throw new ht_io_exception("%s: expression too long (len %d, max %d)", "eval str", d.str.textlen, EVALSTR_MAXSTRLEN);
+		throw ht_io_exception("%s: expression too long (len %d, max %d)", "eval str", d.str.textlen, EVALSTR_MAXSTRLEN);
 	}
 	return request;
 }
@@ -221,7 +221,7 @@ void create_desc_evalstr(char *buf, int buflen, ht_view *f)
 	form->databuf_get(&d, sizeof d);
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "eval str");
+		throw ht_io_exception("%s: string is empty", "eval str");
 	}
 
 	char *b = buf;
@@ -286,7 +286,7 @@ ht_search_request* create_request_vregex(search_pos *start, search_pos *end, ht_
 	ht_regex_search_request *request=NULL;
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "regex");
+		throw ht_io_exception("%s: string is empty", "regex");
 	} else if (d.str.textlen <= VREGEX_MAXSTRLEN) {
 		char strbuf[VREGEX_MAXSTRLEN+1];
 		bin2str(strbuf, d.str.text, d.str.textlen);
@@ -296,7 +296,7 @@ ht_search_request* create_request_vregex(search_pos *start, search_pos *end, ht_
 			request = new ht_regex_search_request(search_class, d.options.state & 1 ? SF_REGEX_CASEINSENSITIVE : 0, strbuf);
 		}
 	} else {
-		throw new ht_io_exception("%s: expression too long (size %d, max %d)", "regex", strlen, VREGEX_MAXSTRLEN);
+		throw ht_io_exception("%s: expression too long (size %d, max %d)", "regex", strlen, VREGEX_MAXSTRLEN);
 	}
 	return request;
 }
@@ -308,7 +308,7 @@ void create_desc_vregex(char *buf, int buflen, ht_view *f)
 	form->databuf_get(&d, sizeof d);
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "regex");
+		throw ht_io_exception("%s: string is empty", "regex");
 	}
 
 	char *b = buf;
@@ -339,7 +339,7 @@ ht_regex_search_request::ht_regex_search_request(UINT search_class, UINT flags,
 {
 	rx_str = ht_strdup(regex);
 	int r = regcomp(&rx, rx_str, REG_EXTENDED | ((flags & SF_REGEX_CASEINSENSITIVE) ? REG_ICASE : 0));
-	if (r) throw new ht_regex_search_exception(r, &rx);
+	if (r) throw ht_regex_search_exception(r, &rx);
 }
 
 ht_regex_search_request::~ht_regex_search_request()
@@ -359,15 +359,12 @@ Object *ht_regex_search_request::duplicate()
  
 ht_regex_search_exception::ht_regex_search_exception(int e, regex_t *r)
 {
-	errorcode=e;
-	regex=r;
-
 	char s[128];
-	regerror(errorcode, regex, s, sizeof s);
+	regerror(e, r, s, sizeof s);
 	ht_snprintf(rxerr, sizeof rxerr, "error compiling regex: %s", s);
 }
 	
-const char* ht_regex_search_exception::what()
+const char* ht_regex_search_exception::what() const
 {
 	return rxerr;
 }
@@ -393,7 +390,7 @@ ht_search_request* create_request_expr(search_pos *start, search_pos *end, ht_vi
 	ht_expr_search_request *request = NULL;
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "expr");
+		throw ht_io_exception("%s: string is empty", "expr");
 	} else if (d.str.textlen <= EXPR_MAXSTRLEN) {
 		char strbuf[EXPR_MAXSTRLEN+1];
 		bin2str(strbuf, d.str.text, d.str.textlen);
@@ -403,7 +400,7 @@ ht_search_request* create_request_expr(search_pos *start, search_pos *end, ht_vi
 			request = new ht_expr_search_request(search_class, 0, strbuf);
 		}
 	} else {
-		throw new ht_io_exception("%s: expression too long (size %d, max %d)", "expr", strlen, EXPR_MAXSTRLEN);
+		throw ht_io_exception("%s: expression too long (size %d, max %d)", "expr", strlen, EXPR_MAXSTRLEN);
 	}
 	return request;
 }
@@ -415,7 +412,7 @@ void create_desc_expr(char *buf, int buflen, ht_view *f)
 	form->databuf_get(&d, sizeof d);
 
 	if (!d.str.textlen) {
-		throw new ht_io_exception("%s: string is empty", "expr");
+		throw ht_io_exception("%s: string is empty", "expr");
 	}
 
 	char *b = buf;
@@ -477,7 +474,7 @@ ht_search_bin_context::~ht_search_bin_context()
 	free(pat);
 }
 
-Object* create_search_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, byte *pat, UINT patlen, UINT flags, UINT *return_ofs, bool *return_success)
+ht_data* create_search_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, byte *pat, UINT patlen, UINT flags, UINT *return_ofs, bool *return_success)
 {
 	if (patlen > SEARCH_BUF_SIZE) return NULL;
 	
@@ -509,7 +506,7 @@ Object* create_search_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, by
 	return ctx;
 }
 
-bool search_bin_process(Object *context, ht_text *progress_indicator)
+bool search_bin_process(ht_data *context, ht_text *progress_indicator)
 {
 	ht_search_bin_context *ctx = (ht_search_bin_context*)context;
 
@@ -537,25 +534,24 @@ bool search_bin_process(Object *context, ht_text *progress_indicator)
 	ctx->o -= ctx->patlen-1;
 
 	int p = (int)(((double)(ctx->o - ctx->ofs))*100/ctx->len);
-	
+
 	char status[64];
 	ht_snprintf(status, sizeof status, "%d %%", p);
 	progress_indicator->settext(status);
-	
+
 	return true;
 }
 
 /*
  *	CLASS ht_hexascii_search_form
  */
-
 void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 {
 	ht_group::init(b, VO_SELECTABLE, NULL);
 	VIEW_DEBUG_NAME("ht_hexascii_search_form");
 
 	bounds c;
-/* ascii string */
+	/* ascii string */
 	c.x=6;
 	c.y=0;
 	c.w=b->w-6;
@@ -563,7 +559,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
 	insert(str);
-/* ascii string label */
+	/* ascii string label */
 	c.x=0;
 	c.w=5;
 	c.h=1;
@@ -571,7 +567,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	strlabel->init(&c, "~ascii", str);
 	insert(strlabel);
 
-/* hex string */
+	/* hex string */
 	c.x=6;
 	c.y=2;
 	c.w=b->w-6;
@@ -580,7 +576,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	hex->init(&c, 64);
 	hex->attach(str);
 	insert(hex);
-/* hex string label */
+	/* hex string label */
 	c.x=0;
 	c.w=5;
 	c.h=1;
@@ -588,7 +584,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	hexlabel->init(&c, "~hex", hex);
 	insert(hexlabel);
 
-/* range start */
+	/* range start */
 	c.x=10;
 	c.y=4;
 	c.w=10;
@@ -596,7 +592,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	range_start=new ht_strinputfield();
 	range_start->init(&c, 10);
 	insert(range_start);
-/* range start label */
+	/* range start label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
@@ -604,7 +600,7 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	rslabel->init(&c, "~from ofs", range_start);
 	insert(rslabel);
 
-/* range end */
+	/* range end */
 	c.x=10;
 	c.y=6;
 	c.w=10;
@@ -612,14 +608,14 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 	range_end=new ht_strinputfield();
 	range_end->init(&c, 10);
 	insert(range_end);
-/* range end label */
+	/* range end label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
 	ht_label *relabel=new ht_label();
 	relabel->init(&c, "~to ofs", range_end);
 	insert(relabel);
-/* options */
+	/* options */
 	c.w=35;
 	c.y=8;
 	c.h=2;
@@ -637,14 +633,13 @@ void ht_hexascii_search_form::init(bounds *b, int options, ht_list *history)
 /*
  *	CLASS ht_evalstr_search_form
  */
-
 void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 {
 	ht_group::init(b, VO_SELECTABLE, NULL);
 	VIEW_DEBUG_NAME("ht_evalstr_search_form");
 
 	bounds c;
-/* string */
+	/* string */
 	c.x=0;
 	c.y=1;
 	c.w=b->w;
@@ -652,7 +647,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
 	insert(str);
-/* string label */
+	/* string label */
 	c.x=0;
 	c.y=0;
 	c.w=23;
@@ -660,7 +655,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	ht_label *strlabel=new ht_label();
 	strlabel->init(&c, "s~earch evaluated string", str);
 	insert(strlabel);
-/* hint */
+	/* hint */
 /*	c.x=0;
 	c.y=2;
 	c.w=b->w-2;
@@ -669,7 +664,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	hint->init(&c, "(example: \"hello\\n\\0\\077\\xd\" 'ho',011b,66o)", 0);
 	insert(hint);*/
 
-/* range start */
+	/* range start */
 	c.x=10;
 	c.y=4;
 	c.w=10;
@@ -677,7 +672,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	range_start=new ht_strinputfield();
 	range_start->init(&c, 10);
 	insert(range_start);
-/* range start label */
+	/* range start label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
@@ -685,7 +680,7 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	rslabel->init(&c, "~from ofs", range_start);
 	insert(rslabel);
 
-/* range end */
+	/* range end */
 	c.x=10;
 	c.y=6;
 	c.w=10;
@@ -693,14 +688,14 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 	range_end=new ht_strinputfield();
 	range_end->init(&c, 10);
 	insert(range_end);
-/* range end label */
+	/* range end label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
 	ht_label *relabel=new ht_label();
 	relabel->init(&c, "~to ofs", range_end);
 	insert(relabel);
-/* options */
+	/* options */
 	c.w = 35;
 	c.y = 8;
 	c.h = 2;
@@ -718,14 +713,13 @@ void ht_evalstr_search_form::init(bounds *b, int options, ht_list *history)
 /*
  *	CLASS ht_vregex_search_form
  */
-
 void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 {
 	ht_group::init(b, VO_SELECTABLE, NULL);
 	VIEW_DEBUG_NAME("ht_text_search_form");
 
 	bounds c;
-/* string */
+	/* string */
 	c.x=0;
 	c.y=1;
 	c.w=b->w;
@@ -733,7 +727,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
 	insert(str);
-/* string label */
+	/* string label */
 	c.x=0;
 	c.y=0;
 	c.w=12;
@@ -742,7 +736,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	strlabel->init(&c, "s~earch regex", str);
 	insert(strlabel);
 
-/* range start */
+	/* range start */
 	c.x=10;
 	c.y=4;
 	c.w=10;
@@ -750,7 +744,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	range_start=new ht_strinputfield();
 	range_start->init(&c, 10);
 	insert(range_start);
-/* range start label */
+	/* range start label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
@@ -758,7 +752,7 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	rslabel->init(&c, "~from addr", range_start);
 	insert(rslabel);
 
-/* range end */
+	/* range end */
 	c.x=10;
 	c.y=6;
 	c.w=10;
@@ -766,14 +760,14 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 	range_end=new ht_strinputfield();
 	range_end->init(&c, 10);
 	insert(range_end);
-/* range end label */
+	/* range end label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
 	ht_label *relabel=new ht_label();
 	relabel->init(&c, "~to addr", range_end);
 	insert(relabel);
-/* options */
+	/* options */
 	c.w = 35;
 	c.y = 8;
 	c.h = 2;
@@ -791,14 +785,13 @@ void ht_vregex_search_form::init(bounds *b, int options, ht_list *history)
 /*
  *	CLASS ht_expr_search_form
  */
-
 void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 {
 	ht_group::init(b, VO_SELECTABLE, NULL);
 	VIEW_DEBUG_NAME("ht_expr_search_form");
 
 	bounds c;
-/* string */
+	/* string */
 	c.x=0;
 	c.y=1;
 	c.w=b->w;
@@ -806,7 +799,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	str=new ht_strinputfield();
 	str->init(&c, 256, history);
 	insert(str);
-/* string label */
+	/* string label */
 	c.x=0;
 	c.y=0;
 	c.w=17;
@@ -814,7 +807,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	ht_label *strlabel=new ht_label();
 	strlabel->init(&c, "s~earch expression", str);
 	insert(strlabel);
-/* hint */
+	/* hint */
 	c.x=0;
 	c.y=2;
 	c.w=b->w-2;
@@ -823,7 +816,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	hint->init(&c, "stops if expression evaluates to non-zero", align_left);
 	insert(hint);
 
-/* range start */
+	/* range start */
 	c.x=10;
 	c.y=4;
 	c.w=10;
@@ -831,7 +824,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	range_start=new ht_strinputfield();
 	range_start->init(&c, 10);
 	insert(range_start);
-/* range start label */
+	/* range start label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
@@ -839,7 +832,7 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	rslabel->init(&c, "~from ofs", range_start);
 	insert(rslabel);
 
-/* range end */
+	/* range end */
 	c.x=10;
 	c.y=6;
 	c.w=10;
@@ -847,14 +840,14 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 	range_end=new ht_strinputfield();
 	range_end->init(&c, 10);
 	insert(range_end);
-/* range end label */
+	/* range end label */
 	c.x=0;
 	c.w=9;
 	c.h=1;
 	ht_label *relabel=new ht_label();
 	relabel->init(&c, "~to ofs", range_end);
 	insert(relabel);
-/* options */
+	/* options */
 /*	c.w=35;
 	c.y=8;
 	c.h=2;
@@ -871,7 +864,6 @@ void	ht_expr_search_form::init(bounds *b, int options, ht_list *history)
 /*
  *	CLASS ht_replace_hexascii_search_form
  */
-
 ht_replace_bin_context::~ht_replace_bin_context()
 {
 	free(repl);
@@ -884,7 +876,7 @@ ht_view* create_form_replace_hexascii(bounds *b, HT_ATOM histid)
 	return form;
 }
 
-Object* create_replace_hexascii_context(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen)
+ht_data* create_replace_hexascii_context(ht_streamfile *file, FILEOFS ofs, UINT len, ht_view *form, UINT *return_repllen)
 {
 	ht_replace_hexascii_search_form_data d;
 	form->databuf_get(&d, sizeof d);
@@ -901,7 +893,7 @@ void ht_replace_hexascii_search_form::init(bounds *b, int options, ht_list *hist
 	VIEW_DEBUG_NAME("ht_replace_hexascii_search_form");
 
 	bounds c;
-/* ascii string */
+	/* ascii string */
 	c.x=6;
 	c.y=0;
 	c.w=40;
@@ -909,7 +901,7 @@ void ht_replace_hexascii_search_form::init(bounds *b, int options, ht_list *hist
 	str=new ht_strinputfield();
 	str->init(&c, 64, history);
 	insert(str);
-/* ascii string label */
+	/* ascii string label */
 	c.x=0;
 	c.w=5;
 	c.h=1;
@@ -917,7 +909,7 @@ void ht_replace_hexascii_search_form::init(bounds *b, int options, ht_list *hist
 	strlabel->init(&c, "~ascii", str);
 	insert(strlabel);
 
-/* hex string */
+	/* hex string */
 	c.x=6;
 	c.y=2;
 	c.w=40;
@@ -926,7 +918,7 @@ void ht_replace_hexascii_search_form::init(bounds *b, int options, ht_list *hist
 	hex->init(&c, 64);
 	hex->attach(str);
 	insert(hex);
-/* hex string label */
+	/* hex string label */
 	c.x=0;
 	c.w=5;
 	c.h=1;
@@ -995,18 +987,18 @@ ht_search_request *search_dialog(ht_format_viewer *format, UINT searchmodes, vie
 		search_pos sstart, send;
 
 		try {
-/* create history entry */
+			/* create history entry */
 			if (s->create_desc) {
 				char hist_desc[1024];
 				s->create_desc(hist_desc, sizeof hist_desc, form);
 				insert_history_entry((ht_list*)find_atom(s->histid), hist_desc, form);
 			}
-/* create request */
-		switch (s->search_class) {
+			/* create request */
+			switch (s->search_class) {
 				case SC_PHYSICAL:
 					if (!format->pos_to_offset(*start, &sstart.offset)
 						|| !format->pos_to_offset(*end, &send.offset)) {
-						throw new ht_io_exception("Internal error: can't convert viewer_pos to offset");
+						throw ht_io_exception("Internal error: can't convert viewer_pos to offset");
 					}
 					break;
 				case SC_VISUAL:
@@ -1019,7 +1011,7 @@ ht_search_request *search_dialog(ht_format_viewer *format, UINT searchmodes, vie
 				case SC_PHYSICAL:
 					if (!format->offset_to_pos(sstart.offset, start)
 					|| !format->offset_to_pos(send.offset, end)) {
-						throw new ht_io_exception("Internal error: can't convert offset to viewer_pos");
+						throw ht_io_exception("Internal error: can't convert offset to viewer_pos");
 					}
 					break;
 				case SC_VISUAL:
@@ -1027,8 +1019,8 @@ ht_search_request *search_dialog(ht_format_viewer *format, UINT searchmodes, vie
 					*end = send.pos;
 					break;
 			}
-		} catch (ht_exception *e) {
-			errorbox("error: %s", e->what());
+		} catch (const ht_exception &e) {
+			errorbox("error: %s", e.what());
 		}
 	}
 	dialog->done();
@@ -1109,20 +1101,20 @@ UINT replace_dialog(ht_format_viewer *format, UINT searchmodes, bool *cancelled)
 		ht_search_request *request = NULL;
 		
 		try {
-/* create history entry */
+			/* create history entry */
 			if (s->create_desc) {
 				char hist_desc[1024];
 				s->create_desc(hist_desc, sizeof hist_desc, sform);
 				insert_history_entry((ht_list*)find_atom(s->histid), hist_desc, sform);
 			}
-/* search */
+			/* search */
 			start.offset=0;
 			end.offset=0xffffffff;
 			format->get_current_offset(&start.offset);
 
 			request = s->create_request(&start, &end, sform, format, s->search_class);
-		} catch (ht_exception *e) {
-			errorbox("error: %s", e->what());
+		} catch (const ht_exception &e) {
+			errorbox("error: %s", e.what());
 		}
 		
 		if (request) {
@@ -1159,7 +1151,7 @@ UINT replace_dialog(ht_format_viewer *format, UINT searchmodes, bool *cancelled)
 					}
 
 					if (replace_all || do_replace) {
-						Object *rctx = r->create_replace_context(format->get_file(), result->offset, result->size, rform, &irepllen);
+						ht_data *rctx = r->create_replace_context(format->get_file(), result->offset, result->size, rform, &irepllen);
 						bool p = execute_process(r->replace_process, rctx);
 						delete rctx;
 						if (!p) {
@@ -1177,8 +1169,8 @@ UINT replace_dialog(ht_format_viewer *format, UINT searchmodes, bool *cancelled)
 					}
 					delete result;
 				}
-			} catch (ht_exception *e) {
-				errorbox("error: %s", e->what());
+			} catch (const ht_exception &e) {
+				errorbox("error: %s", e.what());
 			}
 
 			app->sendmsg(cmd_vstate_restore);
@@ -1192,7 +1184,7 @@ UINT replace_dialog(ht_format_viewer *format, UINT searchmodes, bool *cancelled)
 }
 
 #define REPLACE_COPY_BUF_SIZE	64*1024
-Object* create_replace_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, byte *repl, UINT repllen, UINT *return_repllen)
+ht_data* create_replace_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, byte *repl, UINT repllen, UINT *return_repllen)
 {
 	ht_replace_bin_context *ctx = new ht_replace_bin_context();
 	ctx->file = file;
@@ -1213,13 +1205,13 @@ Object* create_replace_bin_context(ht_streamfile *file, FILEOFS ofs, UINT len, b
 	return ctx;
 }
 
-bool replace_bin_process(Object *context, ht_text *progress_indicator)
+bool replace_bin_process(ht_data *context, ht_text *progress_indicator)
 {
 	progress_indicator->settext("replacing...\n");
 	
 	ht_replace_bin_context *c = (ht_replace_bin_context*)context;
 	if (c->repllen > c->len) {
-/* grow */
+		/* grow */
 		UINT size = c->file->get_size();
 		c->file->extend(size + c->repllen - c->len);
 		
@@ -1235,24 +1227,24 @@ bool replace_bin_process(Object *context, ht_text *progress_indicator)
 		}
 		c->file->seek(c->o);
 		if (c->file->read(c->buf, c->z) != c->z)
-			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+			throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->file->seek(c->o+c->repllen-c->len);
 		if (c->file->write(c->buf, c->z) != c->z)
-			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+			throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 			
 		if (c->o > c->ofs + c->len) return true;
 		
 		c->file->seek(c->ofs);
 		if (c->file->write(c->repl, c->repllen) != c->repllen)
-				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+				throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		free(c->buf);
 	} else if (c->repllen < c->len) {
-/* shrink */
+		/* shrink */
 		UINT size = c->file->get_size();
 		if (c->o == c->ofs + c->len) {
 			c->file->seek(c->ofs);
 			if (c->file->write(c->repl, c->repllen) != c->repllen)
-				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+				throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		}
 		
 		if (c->z > size - c->o) {
@@ -1260,10 +1252,10 @@ bool replace_bin_process(Object *context, ht_text *progress_indicator)
 		}
 		c->file->seek(c->o);
 		if (c->file->read(c->buf, c->z) != c->z)
-			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+			throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->file->seek(c->o - (c->len - c->repllen));
 		if (c->file->write(c->buf, c->z) != c->z)
-			throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+			throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 		c->o += REPLACE_COPY_BUF_SIZE;
 		
 		if (c->z == REPLACE_COPY_BUF_SIZE) return true;
@@ -1273,7 +1265,7 @@ bool replace_bin_process(Object *context, ht_text *progress_indicator)
 	} else {
 		c->file->seek(c->ofs);
 		if (c->file->write(c->repl, c->repllen) != c->repllen)
-				throw new ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
+				throw ht_io_exception("cant replace, write error (ofs=%08x)", c->ofs);
 	}
 	if (c->return_repllen) *c->return_repllen = c->repllen;
 	return false;
@@ -1282,7 +1274,6 @@ bool replace_bin_process(Object *context, ht_text *progress_indicator)
 /*
  *	CLASS ht_search_dialog
  */
-
 void ht_search_dialog::init(bounds *b, const char *title)
 {
 	ht_dialog::init(b, title, FS_KILLER | FS_TITLE | FS_MOVE);
@@ -1389,7 +1380,6 @@ void ht_search_dialog::select_search_mode_bymodeidx()
 /*
  *	CLASS ht_replace_dialog
  */
-
 void ht_replace_dialog::init(bounds *b)
 {
 	ht_search_dialog::init(b, "replace");
@@ -1514,7 +1504,7 @@ ht_search_result *linear_bin_search(ht_search_request *search, FILEOFS start, FI
 		/* create result */
 		bool search_success = false;
 		FILEOFS search_ofs;
-		Object *ctx = create_search_bin_context(file, start, end-start, s->data, s->data_size, fl, &search_ofs, &search_success);
+		ht_data *ctx = create_search_bin_context(file, start, end-start, s->data, s->data_size, fl, &search_ofs, &search_success);
 		if (execute_process(search_bin_process, ctx)) {
 			delete ctx;
 			if (search_success) {
@@ -1527,5 +1517,3 @@ ht_search_result *linear_bin_search(ht_search_request *search, FILEOFS start, FI
 	}
 	return NULL;
 }
-
- 
