@@ -2,7 +2,7 @@
  *	HT Editor
  *	syskeyb.cc - keyboard access functions for POSIX
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -143,9 +143,9 @@ static key_keycode curses_key_defs[] = {
 	{K_Return,		'\n'},
 	{K_Delete,		KEY_DC},
 	{K_Insert,		KEY_IC},
-	{K_BackSpace,		KEY_BACKSPACE},
-	{K_BackSpace,		8},
-	{K_BackSpace,		127},
+	{K_Backspace,		KEY_BACKSPACE},
+	{K_Backspace,		8},
+	{K_Backspace,		127},
 	{K_Alt_Backspace,	META_KEY(KEY_BACKSPACE)},
 	{K_Alt_Backspace,	META_KEY(8)},
 	{K_Alt_Backspace,	META_KEY(127)},
