@@ -2,7 +2,7 @@
  *	HT Editor
  *	out.cc
  *
- *	Copyright (C) 1999-2002 Sebastian Biallas (sb@web-productions.de)
+ *	Copyright (C) 1999-2002 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -38,7 +38,7 @@
 //#define DPRINTF(msg...) printf(##msg)
 //#define DPRINTF2(msg...) printf(##msg)
 
-int compare_keys_addresses_delinear(Object *key_a, Object *key_b)
+int compare_keys_addresses_delinear(ht_data *key_a, ht_data *key_b)
 {
 	return ((Address*)key_a)->compareDelinear((Address*)key_b);
 }
@@ -235,7 +235,7 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 		bool collapsed_xrefs = false;
 		if (analy->mode & ANALY_COLLAPSE_XREFS) {
 			// count xrefs
-			if (xref_count>=(MAX_XREF_COLS*MAX_XREF_LINES)) {
+			if (xref_count >= MAX_XREF_COLS * MAX_XREF_LINES) {
 				collapsed_xrefs = true;
 			}
 		}               
@@ -274,7 +274,7 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 				if (x_tree) {
 					int i=0;
 					AddrXRef *x;
-					Address *xa = (Address*)x_tree->enum_next((Object**)&x, NULL);
+					Address *xa = (Address*)x_tree->enum_next((ht_data**)&x, NULL);
 					while (xa) {
 						if ((i % (MAX_XREF_COLS+1))!=0) {
 							char buf3[90];
@@ -282,7 +282,7 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 							write(buf3);
 							xa->stringify(buf3, 1024, ADDRESS_STRING_FORMAT_COMPACT);
 							write(link(buf3, xa));
-							xa = (Address*)x_tree->enum_next((Object**)&x, xa);
+							xa = (Address*)x_tree->enum_next((ht_data**)&x, xa);
 						} else {
 							if (i!=0) {
 								endLine();
@@ -328,30 +328,39 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 			addr_sym_func_context = this;
 			if (analy->mode & ANALY_TRANSLATE_SYMBOLS) addr_sym_func = &analyser_output_addr_sym_func;
 
-			bool s;
+			int complete_bytes_line = analy->disasm->getSize(o);
+			bool s = true;
 			do {
-				char *x = analy->disasm->str(o, dis_style);
-				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, x);
-				if ((s = analy->disasm->selectNext(o))) {
+				if (s) {
+					char *x = analy->disasm->str(o, dis_style);
+					putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, x);
+				}
+				if (analy->mode & ANALY_EDIT_BYTES) {
+					want_bytes_line = MIN(complete_bytes_line, 16);
+				} else {
+					want_bytes_line = complete_bytes_line;
+				}
+				bytes_line += want_bytes_line;
+				complete_bytes_line -= want_bytes_line;
+				if ((s = analy->disasm->selectNext(o)) || complete_bytes_line) {
 					endLine();
 					beginLine();
 				}
-			} while (s);
+			} while (s || complete_bytes_line);
 			
-			bytes_line += analy->disasm->getSize(o);
 			/* deinits for addr-sym transformations */
 			addr_sym_func_context = NULL;
 			addr_sym_func = NULL;
 		} else {
 			if (buffer_size >= 1) {
 				char buff[20];
-				sprintf(buff, "db      \\@n%02xh", buf[0]);
+				sprintf(buff, "db          \\@n%02xh", buf[0]);
 				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buff);
 				sprintf(buff, " ; '%c'", (buf[0]<32)?' ':buf[0]);
 				putElement(ELEMENT_TYPE_COMMENT, buff);
 
 			} else {
-				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ??");
+				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ??");
 			}
 			bytes_line++;
 		}
@@ -372,21 +381,21 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 								case dst_iword: {
 									word c;
 									analy->bufPtr(addr, (byte *)&c, 2);
-									sprintf(buf, "dw      \\@n%04xh", c);
+									sprintf(buf, "dw          \\@n%04xh", c);
 									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 									break;
 								}
 								case dst_idword: {
 									dword c;
 									analy->bufPtr(addr, (byte *)&c, 4);
-									sprintf(buf, "dd      \\@n%08xh", c);
+									sprintf(buf, "dd          \\@n%08xh", c);
 									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 									break;
 								}
 								case dst_iqword: {
 									qword c;
 									analy->bufPtr(addr, (byte *)&c, 8);
-									ht_snprintf(buf, sizeof buf, "dq      \\@n%016qxh", &c);
+									ht_snprintf(buf, sizeof buf, "dq          \\@n%016qxh", &c);
 									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 									break;
 								}
@@ -394,12 +403,12 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 								default: {
 									byte c;
 									if (analy->bufPtr(addr, (byte *)&c, 1)==1) {
-										sprintf(buf, "db      \\@n%02xh ", c);
+										sprintf(buf, "db          \\@n%02xh ", c);
 										putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 										sprintf(buf, "; '%c'", (c<32)?32:c);
 										putElement(ELEMENT_TYPE_COMMENT, buf);
 									} else {
-										putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ??");
+										putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ??");
 									}
 								}
 							}
@@ -407,14 +416,14 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 							// not initialized
 							switch (cur_addr->type.int_subtype) {
 								case dst_iword:
-									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "dw      ????");
+									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "dw          ????");
 									break;
 								case dst_idword:
-									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "dd      ????????");
+									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "dd          ????????");
 									break;
 								case dst_ibyte:
 								default:
-									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ??");
+									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ??");
 							}
 						}
 						break;
@@ -427,7 +436,7 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 									byte bufread[1024];
 									char *b;
 									int r = analy->bufPtr(addr, bufread, MIN(cur_addr->type.length, 1024));
-									strcpy(buf, "db      \\@s\"");
+									strcpy(buf, "db          \\@s\"");
 									b = buf + 12 + escape_special(buf+12, 100, bufread, r, "\"", false);
 									*b = '\"'; b++; *b = 0;
 									putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
@@ -437,7 +446,7 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 								default: {assert(0);}
 							}
 						} else {
-							putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ?");
+							putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ?");
 						}
 						break;
 					}                         
@@ -451,12 +460,12 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 				byte c;
 				if (analy->validAddress(addr, scinitialized) && (analy->bufPtr(addr, &c, 1)==1)) {
 					char buf[20];
-					sprintf(buf, "db      \\@n%02xh ", c);
+					sprintf(buf, "db          \\@n%02xh ", c);
 					putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 					sprintf(buf, "; '%c'", (c<32)?32:c);
 					putElement(ELEMENT_TYPE_COMMENT, buf);
 				} else {
-					putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ??");
+					putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ??");
 				}
 			}
 		} else {
@@ -466,11 +475,11 @@ void AnalyserOutput::generateAddr(Address *Addr, OutAddr *oa)
 			if (next && next->isValid() && next->difference(d, Addr)) {
 				bytes_line += d;
 				char buf[100];
-				sprintf(buf, "db      ?? * %d", d);
+				sprintf(buf, "db          ?? * %d", d);
 				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, buf);
 			} else {
 				bytes_line += 1;
-				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db      ??");
+				putElement(ELEMENT_TYPE_HIGHLIGHT_DATA_CODE, "db          ??");
 			}
 		}
 	}
@@ -795,10 +804,10 @@ int	AnalyserOutput::prevLine(Address *&Addr, int &line, int n, Address *min)
 			search_line = 0;
 			continue;
 		}
-		assert(i<1024);
 		addrbuf[i & 1023] = DUP_ADDR(search_addr);
 		linebuf[i & 1023] = search_line-1;
 		i++;
+		if (i >= 1023) break;
 	}
 
 	delete next_addr;
