@@ -25,8 +25,8 @@
 #include "htdata.h"
 #include "htobj.h"
 
-typedef bool (*process_func)(Object *context, ht_text *progress_indicator);
+typedef bool (*process_func)(ht_data *context, ht_text *progress_indicator);
 
-bool execute_process(process_func pp, Object *context);
+bool execute_process(process_func pp, ht_data *context);
 
 #endif /* __PROCESS_H__ */
