@@ -34,14 +34,14 @@ protected:
 	bool enabled;
 
 /* new */
-	virtual void	reloc_apply(Object *reloc, byte *data) = 0;
-	virtual bool	reloc_unapply(Object *reloc, byte *data) = 0;
+	virtual void	reloc_apply(ht_data *reloc, byte *data) = 0;
+	virtual bool	reloc_unapply(ht_data *reloc, byte *data) = 0;
 public:
 		   void	init(ht_streamfile *streamfile, bool own_streamfile);
 	virtual void	done();
 /* overwritten */
 		   void	finalize();
-		   void	insert_reloc(FILEOFS o, Object *reloc);
+		   void	insert_reloc(FILEOFS o, ht_data *reloc);
 	virtual UINT	read(void *buf, UINT size);
 	virtual int	vcntl(UINT cmd, va_list vargs);
 	virtual UINT	write(const void *buf, UINT size);
