@@ -174,7 +174,7 @@ OBJECT_ID sym_int_const::object_id() const
  *	CLASS sym_int_token_rec
  */
  
-class sym_int_token_rec: public Object {
+class sym_int_token_rec: public ht_data {
 public:
 	u_op uop;
 	b_op bop;
@@ -858,7 +858,7 @@ void sym_bool_intcmp::simplify()
  *	CLASS sym_bool_token_rec
  */
  
-class sym_bool_token_rec: public Object {
+class sym_bool_token_rec: public ht_data {
 public:
 	n_op nop;
 	l_op lop;
