@@ -358,7 +358,7 @@ void *RegistryFs::enumFiletype(UINT *type, char **name, void *handle)
 	ht_data_string *key = (ht_data_string*)handle;
 	ht_registry_node_type_desc *value;
 	do {
-		key = (ht_data_string *)registry->node_types->enum_next((Object**)&value, key);
+		key = (ht_data_string *)registry->node_types->enum_next((ht_data**)&value, key);
 	} while (value && (value->type==RNT_SUBDIR));
 	if (key && value) {
 		*type=value->type;
